export default class TaskQueue {
    #items = new Set();
    #runningOrScheduled = false;
    #initialDelay;
    #throttleDelay;
    // I'm not sure why the editor needs NodeJS.Timeout here.
    // tsconfig.json doesn't mention that the code needs to run in Node.
    // Maybe it's somehow related to the fact that we're using the Node test runner.
    #timeoutId = null;
    #asyncCallback = null;
    constructor(asyncCallback, { initialDelay, throttleDelay } = {}) {
        this.#asyncCallback = asyncCallback;
        this.#initialDelay = initialDelay ?? 0;
        this.#throttleDelay = throttleDelay ?? 1000;
    }
    #scheduleRun() {
        if (this.#items.size === 0) {
            this.#runningOrScheduled = false;
        }
        if (this.#timeoutId !== null || this.#items.size === 0) {
            return;
        }
        const delay = this.#runningOrScheduled ? this.#throttleDelay : this.#initialDelay;
        this.#runningOrScheduled = true;
        this.#timeoutId = setTimeout(() => this.#run(), delay);
    }
    async #run() {
        this.#items.clear();
        if (this.#asyncCallback) {
            await this.#asyncCallback();
        }
        this.#timeoutId = null;
        this.#scheduleRun();
    }
    add(item) {
        this.#items.add(item);
        this.#scheduleRun();
    }
}
//# sourceMappingURL=task-queue.js.map